﻿using UnityEngine;
using System.Collections;

public class FadeInOut : MonoBehaviour {

	private static FadeInOut _instance = null;

	private Material _material = new Material("Shader \"Plane/No zTest\" { SubShader { Pass { Blend SrcAlpha OneMinusSrcAlpha ZWrite Off Cull Off Fog { Mode Off } BindChannels { Bind \"Color\",color } } } }");
	private bool _fading = false;


	private static FadeInOut instance {
		get{
			if ( _instance == null ) {
				_instance = (new GameObject("FadeInOut")).AddComponent<FadeInOut>();
				DontDestroyOnLoad(_instance);
			}
			return _instance;
		}
	}

	public static bool fading {
		get { return instance._fading; }
		private set { instance._fading = value; }
	}


	private void DrawQuad( Color colour ) {
		//GL stuff
		_material.SetPass(0);
		GL.PushMatrix();
		GL.LoadOrtho();
		GL.Begin( GL.QUADS );
		GL.Color( colour );
		GL.Vertex3( 0, 0, -1 );
		GL.Vertex3( 0, 1, -1 );
		GL.Vertex3( 1, 1, -1 );
		GL.Vertex3( 1, 0, -1 );
		GL.End();
		GL.PopMatrix();
	}

	private IEnumerator Fade( float duration, Color colour, int levelindex ) {
		fading = true;

		float startalpha = 0f;
		float endalpha = 1f;
		float alphadelta = endalpha-startalpha;
		float t = 0f;
		while ( t < duration ) {
			yield return new WaitForEndOfFrame();
			t += Time.deltaTime;
			colour.a = startalpha + alphadelta*(t/duration);
			DrawQuad( colour );
		}

		Application.LoadLevel( levelindex );

		startalpha = 1f;
		endalpha = 0f;
		alphadelta = endalpha-startalpha;
		t = 0f;
		while ( t < duration ) {
			yield return new WaitForEndOfFrame();
			t += Time.deltaTime;
			colour.a = startalpha + alphadelta*(t/duration);
			DrawQuad( colour );
		}

		fading = false;
	}

	public static void ReloadLevel() {
		if( fading ) {
			// if it's already fading, don't reload
			return;
		}
		instance.StartCoroutine( instance.Fade( 0.5f, Color.black, Application.loadedLevel ) );
	}
	public static void NextLevel() {
		if( fading ) {
			// if it's already fading, don't reload
			return;
		}
		// if there's no more levels, loop
		//Debug.Log( Application.levelCount + "  " + Application.loadedLevel+1 );
		if ( Application.levelCount > Application.loadedLevel+1 ) {
			instance.StartCoroutine( instance.Fade( 0.5f, Color.black, Application.loadedLevel+1 ) );
		} else {
			instance.StartCoroutine( instance.Fade( 0.5f, Color.black, 0 ) );
		}
	}

	public static void LoadLevel( int i ) {
		if ( Application.levelCount >= i+1 ) {
			instance.StartCoroutine( instance.Fade( 0.5f, Color.black, i ) );
		}
	}

}





