﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class GuardController : Mover {

	public float viewangle = 35f;
	public float viewdistance = 10f;
	private float alertness = 0;
	private Renderer render;
	private Renderer viewrender;

	private GameObject player;
	private Vector3 lastplayerposition = Vector3.zero;

	private Color mincolour = new Color( 0.9f, 0.88f, 0.25f );
	private Color maxcolour = new Color( 0.56f, 0.13f, 0.13f );
	private Color rewindcolour = new Color( 0.36f, 0.8f, 0.75f );

	public FollowType nodebehaviour;
	public GameObject[] nodes;
	public float[] delays;
	private float currentdelay = 0f;
	private int curindex = 0;
	private int indexdirection = 1;

	private float lookaroundtimer = 2f;
	private float lookaroundnext = 0f;
	private Quaternion lookaroundtarget = Quaternion.identity;

	private float timeflow = 1f;
	private float secondstorewind = 5.5f; // 0.02 timestep -> 50 steps per second

	private List<Vector3> positions = new List<Vector3>();
	private List<Quaternion> rotations = new List<Quaternion>();
	private List<int> indexes = new List<int>();
	private List<int> idirs = new List<int>();
	private List<float> alertnesses = new List<float>();
	private List<Vector3> lastplayerpositions = new List<Vector3>();
	private List<float> previousdelays = new List<float>();
	

	public float TimeFlow{
		get { return timeflow; }
	}

	// Use this for initialization
	protected override void Start () {
		base.Start();
		render = transform.FindChild( "Body").GetComponent<Renderer>();
		viewrender = transform.FindChild( "Viewcone").gameObject.GetComponent<Renderer>();
		player = GameObject.Find( "Player" );


		if ( nodes.Length > 0 ) {
			if ( nodes[0] != null ) {
				Vector3 newpos = nodes[0].transform.position;
				newpos.y = rbody.position.y;
				rbody.position = newpos;
			}
		}

	}
	
	// Update is called once per frame
	void FixedUpdate () {
		if ( !FadeInOut.fading ) {
			if ( timeflow > 0 ) {

				ViewCone();
				alertness = Mathf.Clamp( alertness, 0f, 100f );

				if ( alertness < 30f ) {
					if ( nodes.Length > 0 ) {
						NodeFollow();
					}
				} else {
					if ( alertness < 45f ) {
						//stand still
						//LookAround();
					} else {
						Chase();

					}
				}
				AddStep();
			} else {
				for ( int i = 0; i < Mathf.FloorToInt( timeflow/-1f ); i++ ) {
					StepBackwards();
				}
			}
			ColourChange();
		}
	}


	void ViewCone() {
		//Debug.DrawLine( transform.position, transform.position+ Quaternion.Euler( 0f, viewangle, 0f )*(transform.forward*viewdistance), Color.red );
		//Debug.DrawLine( transform.position, transform.position+ Quaternion.Euler( 0f, -viewangle, 0f )*(transform.forward*viewdistance), Color.red );
		Vector3 playervector = player.transform.position-transform.position;
		if( (playervector).sqrMagnitude < viewdistance*viewdistance ) {
			if ( Vector3.Angle( playervector, transform.forward ) < viewangle ) {
				RaycastHit hit;
				if ( Physics.Raycast( transform.position, playervector.normalized, out hit, viewdistance ) ) {
					if ( hit.collider.gameObject.tag == "Player" ) {
						lastplayerposition = hit.collider.transform.position;
						alertness += 10f;
					}
				}
				//Debug.Log( "PLAYER VISIBLE" );
			} else {
				alertness -= 2f;
			}
		} else {
			alertness -= 2f;
		}
	}


	public void SetTimeFlow( float newflow ) {
		timeflow = newflow;
	}


	void AddStep() {
		positions.Add( rbody.position );
		rotations.Add( rbody.rotation );
		indexes.Add( curindex );
		idirs.Add( indexdirection );
		alertnesses.Add( alertness );
		lastplayerpositions.Add( lastplayerposition );
		previousdelays.Add ( currentdelay );

		if ( positions.Count > Mathf.RoundToInt(50f*secondstorewind)-1 ) {
			positions.RemoveAt(0);
		}
		if ( rotations.Count > Mathf.RoundToInt(50f*secondstorewind)-1 ) {
			rotations.RemoveAt(0);
		}
		if ( indexes.Count > Mathf.RoundToInt(50f*secondstorewind)-1 ) {
			indexes.RemoveAt(0);
		}
		if ( idirs.Count > Mathf.RoundToInt(50f*secondstorewind)-1 ) {
			idirs.RemoveAt(0);
		}
		if ( alertnesses.Count > Mathf.RoundToInt(50f*secondstorewind)-1 ) {
			alertnesses.RemoveAt(0);
		}
		if ( lastplayerpositions.Count > Mathf.RoundToInt(50f*secondstorewind)-1 ) {
			lastplayerpositions.RemoveAt(0);
		}
		if ( previousdelays.Count > Mathf.RoundToInt(50f*secondstorewind)-1 ) {
			previousdelays.RemoveAt(0);
		}
		//Debug.Log( positions[ positions.Count-1 ] + "   " + rotations[  positions.Count-1 ] + "   " + indexes[  positions.Count-1 ] );
	}

	void StepBackwards() {
		//Debug.Log( positions.Count + "   " + rotations.Count + "   " + indexes.Count + "   " + idirs.Count + "   " + alertnesses.Count );
		if ( positions.Count == 0 || rotations.Count == 0 || indexes.Count == 0 || idirs.Count == 0 || alertnesses.Count == 0 || lastplayerpositions.Count == 0 || previousdelays.Count == 0 ) {
			positions.Clear();
			rotations.Clear();
			indexes.Clear();
			idirs.Clear();
			alertnesses.Clear();
			lastplayerpositions.Clear();
			previousdelays.Clear ();

			timeflow = 1f;
		} else {

			if ( positions.Count > 0 ) {
				rbody.position = positions[ positions.Count-1 ];
				positions.RemoveAt( positions.Count-1 );
			} 
			if ( rotations.Count > 0 ) {
				rbody.rotation = rotations[ rotations.Count-1 ];
				rotations.RemoveAt( rotations.Count-1 );
			} 
			if ( indexes.Count > 0 ) {
				curindex = indexes[ indexes.Count-1 ];
				indexes.RemoveAt( indexes.Count-1 );
			} 
			if ( idirs.Count > 0 ) {
				indexdirection = idirs[ idirs.Count-1 ];
				idirs.RemoveAt( idirs.Count-1 );
			} 
			if ( alertnesses.Count > 0 ) {
				alertness = alertnesses[ alertnesses.Count-1 ];
				alertnesses.RemoveAt( alertnesses.Count-1 );
			}
			if ( lastplayerpositions.Count > 0 ) {
				lastplayerposition = lastplayerpositions[ lastplayerpositions.Count-1 ];
				lastplayerpositions.RemoveAt( lastplayerpositions.Count-1 );
			} 
			if ( previousdelays.Count > 0 ) {
				currentdelay = previousdelays[ previousdelays.Count-1 ];
				previousdelays.RemoveAt( previousdelays.Count-1 );
			}
		}
	}


	void NodeFollow() {
		//Debug.Log( curindex );
		if ( nodes[curindex] != null ) {
			Vector3 dir = nodes[curindex].transform.position - rbody.position;
			dir.y = 0;
			if ( dir.sqrMagnitude > 0.2f ) {
				Movement( dir.normalized );
				ViewRotation( dir.normalized );
			} else {
				currentdelay -= Time.fixedDeltaTime;
				if ( currentdelay < 0f ) {
					curindex += indexdirection;
					ClampIndex();
					if ( delays.Length >= curindex+1 ) {
						currentdelay = delays[curindex];
					} else {
						currentdelay = 0f;
					}
				}
			}
		} else {
			curindex += indexdirection;
			ClampIndex();
		}


	}

	void ClampIndex() {
		// make sure we don't spill beyond the array of nodes
		if ( curindex >= nodes.Length ) {
			if ( nodebehaviour == FollowType.Loop ) {
				curindex = 0;
			} else if ( nodebehaviour == FollowType.Reversible ) {
				curindex = nodes.Length-1;
				indexdirection = -1;
			}
		} else if ( curindex < 0 ) {
			if ( nodebehaviour == FollowType.Loop ) {
				curindex = nodes.Length-1;
			} else if ( nodebehaviour == FollowType.Reversible ) {
				curindex = 0;
				indexdirection = 1;
			}
		}
	}


	void Chase() {
		if ( lastplayerposition != Vector3.zero ) {
			Vector3 dir = lastplayerposition - rbody.position;
			dir.y = 0f;
			Movement( dir, 2.5f );
			ViewRotation( dir );
		}
	}

	void LookAround() {
		if ( Time.time > lookaroundnext ) {
			lookaroundtarget = Quaternion.Euler( 0f, 35f, 0f )*rbody.rotation;
			lookaroundnext = Time.time + lookaroundtimer;
		}

		rbody.MoveRotation( Quaternion.Slerp( rbody.rotation, lookaroundtarget, (lookaroundnext-Time.time)/lookaroundtimer ) );

	}

	void ColourChange(){
		Color col = Color.Lerp( mincolour, maxcolour, alertness/100f );
		if ( timeflow < 0 ) {
			col = rewindcolour;
		}
		render.material.SetColor( "_Color", col );
		col.a = 0.10f;
		viewrender.material.SetColor( "_TintColor", col );
	}
}


public enum FollowType {
	Reversible,
	Loop,
}