﻿using UnityEngine;
using System.Collections;

public class Mover : MonoBehaviour {

	protected Rigidbody rbody;
	public float speed = 0.1f;
	private float maxrotang = 20f;

	// Use this for initialization
	protected virtual void Start () {
		rbody = GetComponent<Rigidbody>();
	}

	protected virtual void Movement( Vector3 dir) {
		Movement( dir, 1f );
	}
	protected virtual void Movement( Vector3 dir, float speedmod ) {
		// just in case
		dir.y = 0;
		if ( dir.sqrMagnitude > 1f ) {
			dir.Normalize();
		}
		
		if ( dir.sqrMagnitude > 0.1f ) {
			Vector3 movetarget = transform.position + (dir * speed * speedmod);
			rbody.MovePosition( movetarget );
			
		}
	}

	protected virtual void ViewRotation( Vector3 dir ) {
		// just in case
		dir.y = 0;
		if ( dir.sqrMagnitude > 1f ) {
			dir.Normalize();
		}
		
		if ( dir.sqrMagnitude > 0.1f ) {
			float curang = rbody.rotation.eulerAngles.y;
			float ang = Mathf.Atan2( dir.x, dir.z ) * Mathf.Rad2Deg;
			ang = ClampAngle( ang );
			ang = curang - ang;

			if ( ang > 180 ) {
				ang = -( ang - 180);
			} else if ( ang < -180 ) {
				ang = - (180 + ang );
			}
			
			if ( ang < -maxrotang ) {
				ang = -maxrotang;
			} else if ( ang > maxrotang) {
				ang = maxrotang;
			}
			
			
			Quaternion rottarget = Quaternion.Euler( 0, curang-ang, 0 );
			rbody.MoveRotation( rottarget );
			
		}
	}
	
	
	protected float ClampAngle ( float angle )  {
		if ( angle < 0f ) {
			angle += 360f;
		}
		if ( angle > 360f ) {
			angle -= 360f;
		}
		return Mathf.Clamp (angle, 0f, 360f);
	}


}
