﻿using UnityEngine;
using System.Collections;

public class Projectile : MonoBehaviour {

	private float destroytime;
	private float lifetime = 5f;
	private Transform[] orphans;
	private Rigidbody rbody;

	void Start() {
		rbody = GetComponent<Rigidbody>();
		orphans = gameObject.GetComponentsInChildren<Transform>();
		destroytime = Time.time + lifetime;
		foreach ( Transform t in orphans ) {
			// orphan them all
			t.parent = null;
		}
	}

	void OnTriggerEnter( Collider col ) {
		//Debug.Log( "hit something" );
		if ( col.tag == "Guard" ) {
			col.gameObject.SendMessage( "SetTimeFlow", -5f );
		} 
		if ( col.tag != "Player" ) {
			if ( col.gameObject.layer != 2 ) { // 2 is Ignore Raycast
				Destroy();
			}
		}
	}

	void Update() {


		foreach ( Transform t in orphans ) {
			if ( t != transform ) {
				if ( Random.value < 0.3f ) {
					Vector3 pos = -rbody.velocity.normalized*Random.Range( 0.1f, 1f );
					t.localScale = Vector3.one*0.7f * (1.2f-Mathf.Abs(pos.magnitude));
					pos = Quaternion.Euler( Random.Range( -20f, 20f ), Random.Range( -20f, 20f ), Random.Range( -20f, 20f ) ) * pos;
					t.position = transform.TransformPoint(pos);

				}
			}
		}
		if ( Time.time > destroytime ) {
			Destroy ();
		}
	}

	void Destroy() {

		foreach ( Transform t in orphans ) {
			// kill all the orphans
			if ( t != transform ) {
				t.position = transform.TransformPoint( new Vector3( Random.Range( -1f, 1f ), Random.Range( -1f, 1f ), Random.Range( -1f, 1f ) ));
				Rigidbody r = t.GetComponent<Rigidbody>();
				r.AddForce( Quaternion.Euler( SphereRandom(), Random.Range( 0f, 360f), 0f )*transform.forward*300f );
				r.gameObject.SendMessage("BeginShrink");
			}
		}
		Destroy(gameObject);
		
	}

	float SphereRandom () {
		float Theta = Random.Range( -1f, 1f ); // evenly distributed between 0 and Pi/2
		Theta = Mathf.Acos( Theta );
		Theta *= Mathf.Rad2Deg;
		return Theta-90f;
	}
}
